@@ -366,6 +366,15 @@ def run_qufit(
         specFig.canvas.draw()
         specFig.show()
 
+
+    print("Imodel:"+",".join([str(x.astype(np.float32)) for x in IfitDict["p"]]))
+    print("Imodel_errs:"+",".join([str(x.astype(np.float32)) for x in IfitDict["perror"]]))
+    print("IfitChiSq:"+str(toscalar(IfitDict["chiSq"])))
+    print("IfitChiSqRed:"+str(toscalar(IfitDict["chiSqRed"])))
+    print("IfitPolyOrd:"+str(toscalar(IfitDict["polyOrd"])))
+    print("Ifitfreq0:"+str(toscalar(IfitDict["reference_frequency_Hz"])))
+
+
     # -------------------------------------------------------------------------#
 
     # Load the model and parameters from the relevant file
