@@ -40,6 +40,7 @@ import time
 import math as m
 import numpy as np
 import astropy.io.fits as pf
+import astropy.table as at
 
 from RMutils.util_RM import do_rmsynth_planes
 from RMutils.util_RM import get_rmsf_planes
@@ -53,10 +54,11 @@ if sys.version_info.major == 2:
 C = 2.997924538e8 # Speed of light [m/s]
 
 #-----------------------------------------------------------------------------#
+
 def run_rmsynth(dataQ, dataU, freqArr_Hz, dataI=None, rmsArr=None,
                 phiMax_radm2=None, dPhi_radm2=None, nSamples=10.0,
                 weightType="uniform", fitRMSF=False, nBits=32, verbose=True, not_rmsf = False,
-                log = print):
+                log = print, super_resolution=False):
 
     """Run RM-synthesis on 2/3D data.
 
@@ -115,7 +117,10 @@ def run_rmsynth(dataQ, dataU, freqArr_Hz, dataI=None, rmsArr=None,
     dLambdaSqMax_m2 = np.nanmax(np.abs(np.diff(lambdaSqArr_m2)))
 
     # Set the Faraday depth range
-    fwhmRMSF_radm2 = 2.0 * m.sqrt(3.0) / lambdaSqRange_m2
+    if not super_resolution:
+        fwhmRMSF_radm2 = 3.8 / lambdaSqRange_m2 #Dickey+2019 theoretical RMSF width
+    else: #If super resolution, use R&C23 theoretical width
+        fwhmRMSF_radm2 = 2.0 / (np.nanmax(lambdaSqArr_m2) + np.nanmin(lambdaSqArr_m2))
     if dPhi_radm2 is None:
         dPhi_radm2 = fwhmRMSF_radm2 / nSamples
     if phiMax_radm2 is None:
@@ -160,7 +165,9 @@ def run_rmsynth(dataQ, dataU, freqArr_Hz, dataI=None, rmsArr=None,
                                            phiArr_radm2    = phiArr_radm2,
                                            weightArr       = weightArr,
                                            nBits           = 32,
-                                           verbose         = verbose)
+                                           verbose         = verbose,
+                                           lam0Sq_m2       = 0 if super_resolution else None,
+                                           )
     # Calculate the Rotation Measure Spread Function cube
     if not_rmsf is not True:
         RMSFcube, phi2Arr_radm2, fwhmRMSFCube, fitStatArr = \
@@ -170,8 +177,8 @@ def run_rmsynth(dataQ, dataU, freqArr_Hz, dataI=None, rmsArr=None,
                             mskArr           = ~np.isfinite(dataQ),
                             lam0Sq_m2        = lam0Sq_m2,
                             double           = True,
-                            fitRMSF          = fitRMSF,
-                            fitRMSFreal      = False,
+                            fitRMSF          = fitRMSF or super_resolution,
+                            fitRMSFreal      = super_resolution,
                             nBits            = 32,
                             verbose          = verbose,
                             log              = log)
@@ -185,8 +192,8 @@ def run_rmsynth(dataQ, dataU, freqArr_Hz, dataI=None, rmsArr=None,
     # i.e., no NaNs as the amplitude at freq0_Hz is interpolated from the
     # nearest two planes.
     with np.errstate(divide='ignore', invalid='ignore'):
-        freq0_Hz = np.true_divide(C , m.sqrt(lam0Sq_m2))
-                                  
+        freq0_Hz = np.true_divide(C , m.sqrt(lam0Sq_m2)) if lam0Sq_m2 > 0 else np.nanmean(freqArr_Hz)
+
     if dataI is not None:
         idx = np.abs(freqArr_Hz - freq0_Hz).argmin()
         if freqArr_Hz[idx]<freq0_Hz:
@@ -199,7 +206,6 @@ def run_rmsynth(dataQ, dataU, freqArr_Hz, dataI=None, rmsArr=None,
         # Multiply the dirty FDF by Ifreq0 to recover the PI
         FDFcube *= Ifreq0Arr
 
-
     if not_rmsf:
         dataArr = [FDFcube, phiArr_radm2, lam0Sq_m2, lambdaSqArr_m2]
 
@@ -207,25 +213,12 @@ def run_rmsynth(dataQ, dataU, freqArr_Hz, dataI=None, rmsArr=None,
         dataArr = [FDFcube, phiArr_radm2, RMSFcube, phi2Arr_radm2, fwhmRMSFCube,fitStatArr, lam0Sq_m2, lambdaSqArr_m2]
 
     return dataArr
+    
 
 def writefits(dataArr, headtemplate, fitRMSF=False, prefixOut="", outDir="",
-                nBits = 32, write_seperate_FDF=False, not_rmsf=False, verbose=False, log=print):
+                nBits = 32, write_seperate_FDF=True, not_rmsf=False, verbose=False, log=print):
     """Write data to disk in FITS
 
-
-    Output files:
-        Default:
-        FDF_dirty.fits: FDF, in 3 extensions: Q,U, and PI.
-        FDF_maxPI.fits: 2D map of peak polarized intensity per pixel.
-        FDF_peakRM.fits: 2D map of Faraday depth of highest peak, per pixel.
-        RMSF.fits: 4 extensions; first 3 are RMSF cubes [Q, U, PI]
-                                 4th is 2D map of RMSF FWHM.
-        write_seperate_FDF=True:
-        FDF_dirty.fits is split into three constituent components:
-            FDF_real_dirty.fits: Stokes Q
-            FDF_im_dirty.fits: Stokes U
-            FDF_tot_dirty.fits: Polarizd Intensity (sqrt(Q^2+U^2))
-
     Args:
       dataArr (list): FDF and RMSF information
         if not_rmsf:
@@ -244,6 +237,28 @@ def writefits(dataArr, headtemplate, fitRMSF=False, prefixOut="", outDir="",
         not_rmsf (bool): Just do RM synthesis and ignore RMSF?
         log (function): Which logging function to use.
 
+
+    Output files:
+        Default:
+        FDF_maxPI.fits: 2D map of peak polarized intensity per pixel.
+        FDF_peakRM.fits: 2D map of Faraday depth of highest peak, per pixel.
+
+        write_seperate_FDF=True: [default]
+        FDF_dirty.fits is split into three constituent components:
+            FDF_real_dirty.fits: Stokes Q
+            FDF_im_dirty.fits: Stokes U
+            FDF_tot_dirty.fits: Polarizd Intensity (sqrt(Q^2+U^2))
+            RMSF_real.fits: Real/Stokes Q component of RMSF
+            RMSF_im.fits: Imaginary/Stokes U component of RMSF
+            RM_tot.fits: polarized intensity view of RMSF
+            RMSF_FWHM.fits: 2D map of width of RMSF main lobe
+
+        write_seperate_FDF=False:
+            FDF_dirty.fits: FDF, in 3 extensions: Q,U, and PI.
+            RMSF.fits: 4 extensions; first 3 are RMSF cubes [Q, U, PI]
+                                 4th is 2D map of RMSF FWHM.
+
+
     """
     if not_rmsf:
         FDFcube, phiArr_radm2, lam0Sq_m2, lambdaSqArr_m2 = dataArr
@@ -251,9 +266,10 @@ def writefits(dataArr, headtemplate, fitRMSF=False, prefixOut="", outDir="",
     else:
         FDFcube, phiArr_radm2, RMSFcube, phi2Arr_radm2, fwhmRMSFCube,fitStatArr, lam0Sq_m2, lambdaSqArr_m2 = dataArr
 
-    # Default data types
+    # Default data typess
     dtFloat = "float" + str(nBits)
     dtComplex = "complex" + str(2*nBits)
+    
 
     if(verbose): log("Saving the dirty FDF, RMSF and ancillary FITS files.")
     # Make a copy of the Q header and alter frequency-axis as Faraday depth
@@ -339,7 +355,10 @@ def writefits(dataArr, headtemplate, fitRMSF=False, prefixOut="", outDir="",
     # Save the RMSF
     if not_rmsf is not True:
         header["NAXIS"+str(freq_axis)] = phi2Arr_radm2.size
-        header["CRVAL"+str(freq_axis)] = (phi2Arr_radm2[0],'[rad/m^2] Coordinate value at reference point')
+        header["CDELT"+str(freq_axis)] = (np.diff(phi2Arr_radm2)[0], '[rad/m^2] Coordinate increment at reference point')
+        header["CRPIX"+str(freq_axis)] = phi2Arr_radm2.size//2+1
+        header["CRVAL"+str(freq_axis)] = (phi2Arr_radm2[phi2Arr_radm2.size//2], '[rad/m^2] Coordinate value at reference point')
+
         header["DATAMAX"] = np.max(fwhmRMSFCube) + 1
         header["DATAMIN"] = np.max(fwhmRMSFCube) - 1
         rmheader=header.copy()
@@ -384,8 +403,6 @@ def writefits(dataArr, headtemplate, fitRMSF=False, prefixOut="", outDir="",
             hduLst.writeto(fitsFileOut, output_verify="fix", overwrite=True)
             hduLst.close()
 
-
-
     #Because there can be problems with different axes having different FITS keywords,
     #don't try to remove the FD axis, but just make it degenerate.
     # Also requires np.expand_dims to set the correct NAXIS.
@@ -399,7 +416,7 @@ def writefits(dataArr, headtemplate, fitRMSF=False, prefixOut="", outDir="",
 
 
     #Generate peak maps:
-        
+
     maxPI,peakRM = create_peak_maps(FDFcube,phiArr_radm2,Ndim-freq_axis)
     # Save a maximum polarised intensity map
     fitsFileOut = outDir + "/" + prefixOut + "FDF_maxPI.fits"
@@ -435,7 +452,7 @@ def create_peak_maps(FDFcube,phiArr_radm2,phi_axis=0):
     Inputs:
         FDFcube: output cube from run_rmsynth
         phiArr_radm2: array of Faraday depth values, from run_rmsynth
-        phi_axis (int): number of the axis for Faraday depth (in python order, 
+        phi_axis (int): number of the axis for Faraday depth (in python order,
                          not FITS order). Defaults to zero (first axis).
     Returns:
         maxPI: array of same dimensions as FDFcube exceppt collapsed along
@@ -443,15 +460,15 @@ def create_peak_maps(FDFcube,phiArr_radm2,phi_axis=0):
                 intensity for each pixel
         peakRM: as maxPI, but with the Faraday depth location of the peak
     """
-    
+
     maxPI=np.max(np.abs(FDFcube), axis=phi_axis)
     peakRM_indices = np.argmax(np.abs(FDFcube), axis=phi_axis)
     peakRM=phiArr_radm2[peakRM_indices]
-    
+
     return maxPI, peakRM
 
-    
-    
+
+
 
 
 
@@ -502,7 +519,7 @@ def readFitsCube(file, verbose, log = print):
             print('NAXIS{} = {}'.format(i,head['NAXIS'+str(i)]),end='  ')
         print()
 
-    freq_axis=find_freq_axis(head) 
+    freq_axis=find_freq_axis(head)
     #If the frequency axis isn't the last one, rotate the array until it is.
     #Recall that pyfits reverses the axis ordering, so we want frequency on
     #axis 0 of the numpy array.
@@ -575,7 +592,7 @@ def main():
     parser.add_argument("-n", dest="noiseFile", default=None,
                         help="Text file containing channel noise values [None].")
     parser.add_argument("-w", dest="weightType", default="uniform",
-                        help="weighting [uniform] (all 1s) or 'variance'.")
+                        help="weighting ['uniform'] (all 1s) or 'variance'.")
     parser.add_argument("-t", dest="fitRMSF", action="store_true",
                         help="Fit a Gaussian to the RMSF [False]")
     parser.add_argument("-l", dest="phiMax_radm2", type=float, default=None,
@@ -587,11 +604,14 @@ def main():
     parser.add_argument("-s", dest="nSamples", type=float, default=5,
                         help="Number of samples across the FWHM RMSF.")
     parser.add_argument("-f", dest="write_seperate_FDF", action="store_false",
-                        help="Store different Stokes as FITS extensions [False, store as seperate files].")
+                        help="Store different Stokes as FITS extensions [False, store as separate files].")
     parser.add_argument("-v", dest="verbose", action="store_true",
                         help="Verbose [False].")
     parser.add_argument("-R", dest="not_RMSF", action="store_true",
                         help="Skip calculation of RMSF? [False]")
+    parser.add_argument("-r", "--super-resolution", action="store_true",
+                        help="Optimise the resolution of the RMSF (as per Rudnick & Cotton). "
+                        )
     args = parser.parse_args()
 
     # Sanity checks
@@ -625,7 +645,9 @@ def main():
                           fitRMSF      = args.fitRMSF,
                           nBits        = 32,
                           verbose      = verbose,
-                          not_rmsf = args.not_RMSF)
+                          not_rmsf = args.not_RMSF,
+                          super_resolution=args.super_resolution,
+                        )
 
     # Write to files
     writefits(dataArr,
